"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processor = void 0;
const gatsby_source_filesystem_1 = require("gatsby-source-filesystem");
const metadataProcessor_1 = require("./metadataProcessor");
const tableOfContent_1 = require("./tableOfContent");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const processor = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const { thumbnail, tableOfContents, updatedBlocks, rawText } = await processBlocksForContent(blocks, actions, getCache, createNodeId, reporter, cache);
    await (0, metadataProcessor_1.processMetadata)(blocks, actions, createNodeId, reporter, cache);
    return [thumbnail, tableOfContents, updatedBlocks, rawText];
};
exports.processor = processor;
const processBlocksForContent = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const tableOfContents = [];
    let thumbnail = null;
    let rawText = "";
    const updatedBlocks = [];
    let firstImageBlock = null;
    for (const block of blocks) {
        if (isImageBlock(block)) {
            firstImageBlock = block;
            break;
        }
    }
    if (firstImageBlock) {
        const updatedBlock = await processImageBlock(firstImageBlock, actions, getCache, createNodeId, reporter, cache);
        if (updatedBlock?.image?.fileId) {
            thumbnail = updatedBlock.image.fileId;
        }
        updatedBlocks[blocks.indexOf(firstImageBlock)] =
            updatedBlock || firstImageBlock;
    }
    await Promise.all(blocks.map(async (block, index) => {
        if (block === firstImageBlock)
            return;
        await (0, tableOfContent_1.processTableOfContents)(block, tableOfContents);
        const plainText = extractPlainText(block);
        if (plainText) {
            rawText += plainText + " ";
        }
        if (isImageBlock(block)) {
            const updatedBlock = await processImageBlock(block, actions, getCache, createNodeId, reporter, cache);
            updatedBlocks[index] = updatedBlock || block;
        }
        else {
            updatedBlocks[index] = block;
        }
    }));
    return { thumbnail, tableOfContents, updatedBlocks, rawText };
};
const isTextContentBlock = (block) => {
    return [
        "paragraph",
        "heading_1",
        "heading_2",
        "heading_3",
        "quote",
        "bulleted_list_item",
        "numbered_list_item",
        "callout",
        "code",
    ].includes(block.type);
};
const extractPlainText = (block) => {
    if (isTextContentBlock(block)) {
        const richTextArray = block[block.type]?.rich_text || [];
        return richTextArray
            .map((text) => block.type === "code" // code의 \n 제거
            ? text.plain_text.replace(/\\n/g, "")
            : text.plain_text)
            .join(" ");
    }
    return null;
};
const isImageBlock = (block) => {
    return block.type === "image" && "image" in block;
};
const processImageBlock = async (block, actions, getCache, createNodeId, reporter, cache) => {
    const { createNode } = actions;
    if (block.type === "image" && "image" in block) {
        const imageSourceType = block.image.type;
        const imageUrl = imageSourceType === `external`
            ? block.image.external?.url
            : block.image?.file?.url;
        if (!imageUrl)
            return null;
        // GIF 파일 처리
        if (imageUrl.endsWith(".gif")) {
            const staticDir = path_1.default.join(process.cwd(), "static"); // Gatsby의 static 디렉토리
            const gifFileName = path_1.default.basename(imageUrl); // 파일 이름 추출
            const gifFilePath = path_1.default.join(staticDir, gifFileName);
            // 이미 static 디렉토리에 파일이 있는지 확인
            if (!fs_extra_1.default.existsSync(gifFilePath)) {
                try {
                    reporter.info(`[GIF PROCESSING] Downloading GIF: ${imageUrl}`);
                    const response = await fetch(imageUrl);
                    if (!response.ok) {
                        throw new Error(`Failed to download GIF: ${response.statusText}`);
                    }
                    const arrayBuffer = await response.arrayBuffer();
                    const buffer = Buffer.from(arrayBuffer);
                    await fs_extra_1.default.ensureDir(staticDir); // static 디렉토리 생성
                    await fs_extra_1.default.writeFile(gifFilePath, buffer); // GIF 파일 저장
                    reporter.info(`[GIF SUCCESS] Saved GIF to static: ${gifFilePath}`);
                }
                catch (error) {
                    reporter.warn(`[GIF WARNING] Failed to process GIF: ${imageUrl}`);
                    return null;
                }
            }
            else {
                reporter.info(`[GIF CACHE HIT] GIF already exists: ${gifFilePath}`);
            }
            // GIF 파일을 정적 파일로 추가
            const updatedBlock = {
                ...block,
                image: {
                    fileId: gifFileName, // static 경로를 기준으로 참조
                    caption: block.image.caption,
                },
            };
            return updatedBlock;
        }
        // GIF가 아닌 경우 기존 로직 유지
        const cacheKey = `${imageUrl}-post-image`;
        const cachedFileNodeId = await cache.get(cacheKey);
        if (cachedFileNodeId) {
            reporter.info(`[CACHE HIT] Image already processed: ${imageUrl}`);
            const updatedBlock = {
                ...block,
                image: {
                    fileId: cachedFileNodeId,
                    caption: block.image.caption,
                },
            };
            return updatedBlock;
        }
        try {
            const fileNode = await (0, gatsby_source_filesystem_1.createRemoteFileNode)({
                url: imageUrl,
                parentNodeId: block.id,
                getCache,
                createNode,
                createNodeId,
            });
            if (fileNode) {
                const updatedBlock = {
                    ...block,
                    image: {
                        fileId: fileNode.id,
                        caption: block.image.caption,
                    },
                };
                reporter.info(`[SUCCESS] Image processed: ${fileNode.id}`);
                await cache.set(cacheKey, fileNode.id);
                return updatedBlock;
            }
        }
        catch (error) {
            reporter.warn(`[WARNING] Failed to download image: ${imageUrl}`);
        }
    }
    return null;
};
